import { type Message } from "@shared/schema";
import { Skeleton } from "@/components/ui/skeleton";
import { AlertCircle, RefreshCw, Copy, CheckCircle2, Check, Clock, MessageSquare, CornerDownRight, Bot, Image } from "lucide-react";
import { format, formatDistanceToNow, isToday, isYesterday } from "date-fns";
import ReactMarkdown from "react-markdown";
import { Prism as SyntaxHighlighter } from "react-syntax-highlighter";
import { vscDarkPlus } from "react-syntax-highlighter/dist/esm/styles/prism";
import { motion, AnimatePresence } from "framer-motion";
import { useState, useRef } from "react";
import { Button } from "@/components/ui/button";
import ScrollToBottom from "react-scroll-to-bottom";
import { Tooltip, TooltipProvider, TooltipTrigger, TooltipContent } from "@/components/ui/tooltip";
import { cn } from "@/lib/utils";
import { type ModelType } from "@shared/schema";
import { Wallet } from "lucide-react";
import { ethers } from "ethers";
import { useToast } from "@/hooks/use-toast";
import { SiOpenai } from "react-icons/si";
import { MistralLogo } from "@/components/icons/MistralLogo";
import { AnthropicLogo } from "@/components/icons/AnthropicLogo";
import { QwenLogo } from "@/components/icons/QwenLogo";
import { DeepseekLogo } from "@/components/icons/DeepseekLogo";
import { XAILogo } from "@/components/icons/XAILogo";

// Model icons for assistant messages
const modelIcons: Record<ModelType, React.ReactNode> = {
  "gpt-4o-2024-11-20": <SiOpenai className="h-5 w-5" />,
  "claude-3-7-sonnet": <AnthropicLogo className="h-5 w-5" />,
  "claude-3-5-sonnet-20241022": <AnthropicLogo className="h-5 w-5" />,
  "qwen-2.5-max": <QwenLogo className="h-5 w-5" />,
  "deepseek-r1": <DeepseekLogo className="h-5 w-5" />,
  "mistral-small-3.1-24b": <MistralLogo className="h-5 w-5" />,
  "grok-3-mini-beta": <XAILogo className="h-5 w-5" />,
  "image-generator": <Image className="h-5 w-5" />,
  "background-remover": <Image className="h-5 w-5" />,
  "contract-audit": <Bot className="h-5 w-5" />,
  "deployed-contract-audit": <Bot className="h-5 w-5" />
};

// Language names for syntax highlighting
const SUPPORTED_LANGUAGES = {
  js: 'javascript',
  jsx: 'jsx',
  ts: 'typescript',
  tsx: 'typescript',
  py: 'python',
  python: 'python',
  rb: 'ruby',
  go: 'go',
  rust: 'rust',
  java: 'java',
  cpp: 'cpp',
  c: 'c',
  cs: 'csharp',
  php: 'php',
  html: 'html',
  css: 'css',
  json: 'json',
  yaml: 'yaml',
  sql: 'sql',
  sh: 'bash',
  bash: 'bash',
  shell: 'bash',
  md: 'markdown',
  markdown: 'markdown',
} as const;

interface MessageListProps {
  messages: Message[];
  loading?: boolean;
  error?: string;
  onRetry?: () => void;
  hasStartedTyping?: boolean;
  model: ModelType;
  chatId: number;
}

function MessageGroup({ date }: { date: Date }) {
  let label = "";
  if (isToday(date)) {
    label = "Today";
  } else if (isYesterday(date)) {
    label = "Yesterday";
  } else {
    label = format(date, "MMMM d, yyyy");
  }

  return (
    <div className="flex items-center justify-center my-6">
      <div className="text-xs text-muted-foreground bg-background/80 px-2 py-1 rounded-full border">
        {label}
      </div>
    </div>
  );
}

function TypingIndicator() {
  return (
    <div className="flex items-center gap-1">
      <motion.span
        className="w-1 h-1 bg-current rounded-full"
        animate={{ scale: [1, 1.2, 1] }}
        transition={{ repeat: Infinity, duration: 1, repeatDelay: 0.2 }}
      />
      <motion.span
        className="w-1 h-1 bg-current rounded-full"
        animate={{ scale: [1, 1.2, 1] }}
        transition={{ repeat: Infinity, duration: 1, delay: 0.2, repeatDelay: 0.2 }}
      />
      <motion.span
        className="w-1 h-1 bg-current rounded-full"
        animate={{ scale: [1, 1.2, 1] }}
        transition={{ repeat: Infinity, duration: 1, delay: 0.4, repeatDelay: 0.2 }}
      />
    </div>
  );
}

export function MessageList({ messages, loading, error, onRetry, hasStartedTyping, model, chatId }: MessageListProps) {
  const { toast } = useToast();
  const [copiedId, setCopiedId] = useState<number | null>(null);
  const [autoScroll, setAutoScroll] = useState(true);
  const scrollRef = useRef<HTMLDivElement>(null);

  const handleCopy = async (text: string, id: number) => {
    await navigator.clipboard.writeText(text);
    setCopiedId(id);
    setTimeout(() => setCopiedId(null), 2000);
  };

  const getErrorSuggestion = (error: string) => {
    if (error.includes("Daily Limit Reached")) {
      return "";
    }
    if (error.includes("API Error")) {
      return "Try selecting a different model or waiting a moment before retrying.";
    }
    if (error.includes("Failed to fetch")) {
      return "Check your internet connection and try again.";
    }
    return "";
  };

  const messageGroups = messages.reduce((groups: { [key: string]: Message[] }, message) => {
    const date = format(new Date(message.createdAt), "yyyy-MM-dd");
    if (!groups[date]) {
      groups[date] = [];
    }
    groups[date].push(message);
    return groups;
  }, {});

  const renderMessage = (message: Message, index: number, groupMessages: Message[]) => {
    const isAssistant = message.role === "assistant";
    const isThreadStart = index === 0 || groupMessages[index - 1].role !== message.role;
    const isThreadEnd = index === groupMessages.length - 1 || groupMessages[index + 1].role !== message.role;

    const messageContent = (
      <div
        className={cn(
          "relative flex-1 max-w-[95%] xs:max-w-[90%] sm:max-w-[80%] rounded-lg transition-colors overflow-hidden",
          message.role === "user"
            ? "bg-[#1A1D24] text-neutral-100"
            : "bg-[#0D1117] hover:bg-[#0D1117]/90 text-neutral-100"
        )}
      >
        {!isThreadStart && (
          <div className="absolute -top-4 left-4">
            <CornerDownRight className="h-3 w-3 sm:h-4 sm:w-4 text-neutral-400/50" />
          </div>
        )}

        <TooltipProvider>
          <Tooltip>
            <TooltipTrigger asChild>
              <Button
                variant="ghost"
                size="icon"
                className="absolute right-1 sm:right-2 top-1 sm:top-2 opacity-0 group-hover:opacity-20 hover:opacity-100 transition-opacity h-6 w-6 sm:h-8 sm:w-8 text-neutral-400 hover:text-neutral-100"
                onClick={() => handleCopy(message.content, message.id)}
              >
                {copiedId === message.id ? (
                  <CheckCircle2 className="h-3 w-3 sm:h-4 sm:w-4" />
                ) : (
                  <Copy className="h-3 w-3 sm:h-4 sm:w-4" />
                )}
              </Button>
            </TooltipTrigger>
            <TooltipContent>Copy message</TooltipContent>
          </Tooltip>
        </TooltipProvider>

        <div className="px-3 sm:px-4 pt-3 sm:pt-4 pb-2">
          {/* Process the message content to simplify file extraction display */}
          {(() => {
            // Function to process the message content for display
            const processMessageForDisplay = (content: string): string => {
              // Check if this is a message with file uploads
              if (content.includes('📎 File Uploaded:') && content.includes('📝 Extracted Content:')) {
                // Split the message into sections based on file markers
                const sections = content.split('📎 File Uploaded:');
                
                // The first section is any text before file uploads
                const userText = sections[0].trim();
                
                // Process each file section
                const processedSections = sections.slice(1).map(section => {
                  // Get the file name from the section
                  const fileNameMatch = section.split('\n')[0].trim();
                  
                  // Check if there's extracted content in this section
                  if (section.includes('📝 Extracted Content:')) {
                    // The section after "📝 Extracted Content:" contains the actual extracted text
                    const extractedContentParts = section.split('📝 Extracted Content:');
                    const extractedContent = extractedContentParts[1].trim();
                    
                    // Check if extraction failed based on common error messages
                    const extractionFailed = 
                      extractedContent.includes('[This appears to be') || 
                      extractedContent.includes('[PDF document was uploaded') ||
                      extractedContent.includes('[Note: Could not extract');
                    
                    // Get file type for display
                    let fileType = "file";
                    if (extractedContent.includes('PDF document')) fileType = "PDF";
                    else if (extractedContent.includes('Image OCR results')) fileType = "image";
                    else if (extractedContent.includes('DOCX document')) fileType = "document";
                    else if (extractedContent.includes('CSV file')) fileType = "CSV";
                    else if (extractedContent.includes('JSON content')) fileType = "JSON";
                    else if (extractedContent.includes('ZIP archive')) fileType = "ZIP";
                    
                    // Show simplified success/failure indicator
                    const successMessage = extractionFailed
                      ? `📝 Extracted Content: ❌ No text could be extracted.`
                      : `📝 Extracted Content: ✓ Text successfully extracted from ${fileType}.`;
                    
                    // Return the file name and success message
                    return `📎 File Uploaded: ${fileNameMatch}\n${successMessage}`;
                  }
                  
                  // If no extracted content marker, just return the file name
                  return `📎 File Uploaded: ${fileNameMatch}`;
                });
                
                // Combine the user text with processed file sections
                if (userText) {
                  return `${userText}\n\n${processedSections.join('\n')}`;
                } else {
                  return processedSections.join('\n');
                }
              }
              
              // If not a file upload message, return the original content
              return content;
            };
            
            // Get the processed content for display
            const displayContent = processMessageForDisplay(message.content);
            
            return (
              <ReactMarkdown
                components={{
                  code({ node, className, children, ...props }) {
                    const match = /language-(\w+)/.exec(className || "");
                    const lang = match && match[1];
                    const language = lang && SUPPORTED_LANGUAGES[lang.toLowerCase() as keyof typeof SUPPORTED_LANGUAGES];

                    return !language ? (
                      <code {...props} className="bg-[#0D1117]/50 rounded px-1.5 py-0.5 text-xs sm:text-sm text-neutral-200">
                        {children}
                      </code>
                    ) : (
                      <div className="relative group my-3 sm:my-4">
                        <div className="absolute right-1 sm:right-2 top-1 sm:top-2 opacity-0 group-hover:opacity-20 hover:opacity-100 transition-opacity">
                          <Button
                            variant="ghost"
                            size="icon"
                            className="h-5 w-5 sm:h-6 sm:w-6 text-neutral-400 hover:text-neutral-100"
                            onClick={() => handleCopy(String(children).replace(/\n$/, ""), message.id)}
                          >
                            {copiedId === message.id ? (
                              <CheckCircle2 className="h-2.5 w-2.5 sm:h-3 sm:w-3" />
                            ) : (
                              <Copy className="h-2.5 w-2.5 sm:h-3 sm:w-3" />
                            )}
                          </Button>
                        </div>
                        <div className="text-[10px] sm:text-xs text-neutral-400 mb-1 px-3 sm:px-4 py-1 bg-[#0D1117]/50 rounded-t-md border-b border-neutral-800">
                          {language}
                        </div>
                        <SyntaxHighlighter
                          style={vscDarkPlus}
                          language={language}
                          PreTag="div"
                          className="!my-0 !bg-[#0D1117]/50 !rounded-t-none !rounded-b-md !text-[11px] sm:!text-sm"
                        >
                          {String(children).replace(/\n$/, "")}
                        </SyntaxHighlighter>
                      </div>
                    );
                  },
                  p({ children }) {
                    return <p className="mb-3 sm:mb-4 last:mb-0 leading-6 sm:leading-7 text-sm sm:text-base text-neutral-100">{children}</p>;
                  },
                  ul({ children }) {
                    return <ul className="list-disc list-inside mb-3 sm:mb-4 space-y-1.5 sm:space-y-2 text-sm sm:text-base text-neutral-100">{children}</ul>;
                  },
                  ol({ children }) {
                    return <ol className="list-decimal list-inside mb-3 sm:mb-4 space-y-1.5 sm:space-y-2 text-sm sm:text-base text-neutral-100">{children}</ol>;
                  },
                  li({ children }) {
                    return <li className="leading-6 sm:leading-7 ml-3 sm:ml-4 text-sm sm:text-base text-neutral-100">{children}</li>;
                  },
                  h1({ children }) {
                    return <h1 className="text-2xl sm:text-2xl font-bold mb-4 mt-6 text-neutral-100">{children}</h1>;
                  },
                  h2({ children }) {
                    return <h2 className="text-xl sm:text-xl font-bold mb-3 mt-5 text-neutral-100">{children}</h2>;
                  },
                  h3({ children }) {
                    return <h3 className="text-lg sm:text-lg font-bold mb-3 mt-4 text-neutral-100">{children}</h3>;
                  },
                  blockquote({ children }) {
                    return <blockquote className="border-l-4 border-neutral-700 pl-4 my-4 italic text-neutral-300">{children}</blockquote>;
                  },
                  hr() {
                    return <hr className="my-4 border-neutral-800" />;
                  }
                }}
                className="prose prose-invert max-w-none prose-p:leading-7 prose-pre:my-0"
              >
                {displayContent}
              </ReactMarkdown>
            );
          })()}
          
        </div>

        <div className="border-t border-neutral-800 px-4 py-2 mt-2">
          <div className="flex items-center gap-2 text-xs text-neutral-400">
            <Tooltip>
              <TooltipTrigger asChild>
                <time className="flex items-center gap-1 cursor-help">
                  <Clock className="h-3 w-3" />
                  {format(new Date(message.createdAt), "h:mm a")}
                </time>
              </TooltipTrigger>
              <TooltipContent>
                {format(new Date(message.createdAt), "PPpp")}
              </TooltipContent>
            </Tooltip>
            {message.role === "assistant" && (
              <div className="flex items-center gap-1">
                <Check className="h-3 w-3" />
                delivered
              </div>
            )}
          </div>
        </div>
      </div>
    );

    if (!isAssistant) {
      return (
        <div
          key={`${message.id}-${message.role}`}
          className={cn(
            "group flex items-start gap-4 mb-4",
            message.role === "user" ? "justify-end" : "justify-start"
          )}
        >
          {messageContent}
        </div>
      );
    }

    return (
      <motion.div
        key={`${message.id}-${message.role}`}
        initial={{ opacity: 0, y: 10 }}
        animate={{ opacity: 1, y: 0 }}
        transition={{
          duration: 0.4,
          ease: "easeOut"
        }}
        className={cn(
          "group flex items-start gap-4 mb-4",
          "justify-start"
        )}
      >
        <div className="w-8 h-8 sm:w-10 sm:h-10 rounded-full bg-[#0D1117] border border-neutral-800 flex items-center justify-center overflow-hidden text-neutral-200">
          {modelIcons[model]}
        </div>
        {messageContent}
      </motion.div>
    );
  };

  const handlePayment = async () => {
    try {
      if (!window.ethereum) {
        toast({
          variant: "destructive",
          title: "Wallet not found",
          description: "Please install MetaMask or another Web3 wallet to continue.",
        });
        return;
      }

      const provider = new ethers.BrowserProvider(window.ethereum);
      const signer = await provider.getSigner();

      const transaction = {
        to: "0xD05D192FB8d85E8aEC93973f930f6428807d28A1",
        value: ethers.parseEther("0.001"),
      };

      const tx = await signer.sendTransaction(transaction);

      toast({
        title: "Transaction Sent",
        description: "Please wait for the transaction to be confirmed. Your unlimited access will be activated automatically.",
      });

      await tx.wait();

      toast({
        title: "Transaction Confirmed",
        description: "Your payment has been confirmed. Unlimited access will be activated shortly.",
      });

      if (onRetry) {
        onRetry();
      }
    } catch (error: any) {
      console.error('Transaction error:', error);
      toast({
        title: "Transaction Failed",
        description: error.message || "Failed to send payment. Please try again.",
        variant: "destructive",
      });
    }
  };

  return (
    <TooltipProvider>
      <div className="relative flex-1 h-full">
        <AnimatePresence>
          {!hasStartedTyping && (
            <motion.div
              initial={{ opacity: 0.8, y: 20 }}
              animate={{ opacity: 0.8, y: 0 }}
              exit={{ opacity: 0, y: -20 }}
              className="absolute inset-0 flex items-center justify-center z-10 pointer-events-none"
            >
              <div className="text-muted-foreground text-center bg-background/40 backdrop-blur-sm px-6 py-3 rounded-lg">
                <MessageSquare className="h-5 w-5 mx-auto mb-2 opacity-50" />
                Send a message to start the conversation
              </div>
            </motion.div>
          )}
        </AnimatePresence>

        <ScrollToBottom
          className="flex-1 h-full overflow-x-hidden"
          followButtonClassName="hidden"
          initialScrollBehavior="auto"
          mode={autoScroll ? "bottom" : "top"}
        >
          <div className="max-w-3xl mx-auto space-y-6 px-4 pb-10 sm:pb-20">
            <AnimatePresence mode="sync">
              {Object.entries(messageGroups).map(([date, groupMessages]) => (
                <motion.div
                  key={date}
                  initial={{ opacity: 0 }}
                  animate={{ opacity: 1 }}
                  transition={{ duration: 0.2 }}
                >
                  <MessageGroup date={new Date(date)} />
                  {groupMessages.map((message, index) => renderMessage(message, index, groupMessages))}
                </motion.div>
              ))}
            </AnimatePresence>

            <AnimatePresence>
              {loading && (
                <motion.div
                  initial={{ opacity: 0, y: 20 }}
                  animate={{ opacity: 1, y: 0 }}
                  exit={{ opacity: 0, y: -20 }}
                  transition={{ duration: 0.2 }}
                  className="flex items-start gap-4 px-4 mb-4"
                >
                  <div className="w-8 h-8 sm:w-10 sm:h-10 rounded-full bg-[#0D1117] border border-neutral-800 flex items-center justify-center overflow-hidden text-neutral-200">
                    {modelIcons[model]}
                  </div>
                  <div className="flex-1 max-w-[60%] space-y-3">
                    <div className="bg-[#0D1117] rounded-lg p-4">
                      <div className="flex items-center gap-2 text-sm text-muted-foreground">
                        <TypingIndicator />
                        <span>AI is typing</span>
                      </div>
                    </div>
                  </div>
                </motion.div>
              )}
              {error && (
                <motion.div
                  initial={{ opacity: 0, scale: 0.95 }}
                  animate={{ opacity: 1, scale: 1 }}
                  exit={{ opacity: 0, scale: 0.9 }}
                  className="flex justify-center px-4 mb-4"
                >
                  <div className="flex flex-col items-center gap-2 bg-destructive/10 px-6 py-4 rounded-lg text-sm max-w-md w-full">
                    <div className="flex items-center gap-2 text-destructive">
                      <AlertCircle className="h-4 w-4" />
                      <span>{error}</span>
                    </div>
                    {error.includes("Daily Limit Reached") ? (
                      <div className="space-y-2 w-full pt-4">
                        <Button
                          onClick={() => window.open('https://cloud.google.com/application/web3/faucet/ethereum/sepolia', '_blank')}
                          className="w-full h-10 bg-background hover:bg-accent hover:text-accent-foreground transition-colors"
                          variant="outline"
                        >
                          <Wallet className="mr-2 h-4 w-4" />
                          Get SepoliaETH (Buy & Hold)
                        </Button>
                        <Button
                          onClick={handlePayment}
                          className="w-full h-10 bg-background hover:bg-accent hover:text-accent-foreground transition-colors"
                          variant="outline"
                        >
                          <Wallet className="mr-2 h-4 w-4" />
                          Buy 30 Days Access
                        </Button>
                      </div>
                    ) : (
                      <p className="text-muted-foreground text-xs text-center">
                        {getErrorSuggestion(error)}
                      </p>
                    )}
                  </div>
                </motion.div>
              )}
            </AnimatePresence>
          </div>
        </ScrollToBottom>
      </div>
    </TooltipProvider>
  );
}